from pureauth_pysdk.dto import PureAUTHServerConfigDTO

from .v1 import V1
from .v2 import V2


class Sync:
    def __init__(self, config: PureAUTHServerConfigDTO):
        self.config = config
        self.v1 = V1(config=config)
        self.v2 = V2(config=config)



import base64
import hashlib
import re

import ecdsa
from .errors import PrivateKeyParseError
from ecdsa.keys import BadSignatureError


def format_private_key(private_key):
    private_key = private_key.strip()
    x = re.sub(r"\n*-----[A-Z]* [A-Z]* [A-Z]* [A-Z]*-----\n*", "", private_key)
    pr_key = str(x)
    return pr_key


def sign_dataset(private_key: str, dataset: str):
    formatted_pk = format_private_key(private_key)
    sk = ecdsa.SigningKey.from_der(base64.b64decode(formatted_pk))
    dataset = dataset.encode("utf-8")
    sig = sk.sign(dataset, hashfunc=hashlib.sha256, sigencode=ecdsa.util.sigencode_der)
    return base64.b64encode(sig).decode("utf-8")


def sign_dataset_using_privatekey_file(private_key_path: str, dataset: str):
    with open(private_key_path) as file:
        sk = ecdsa.SigningKey.from_pem(file.read())
    dataset = dataset.encode("utf-8")
    sig = sk.sign(dataset, hashfunc=hashlib.sha256, sigencode=ecdsa.util.sigencode_der)
    return base64.b64encode(sig).decode("utf-8")


def employee_id(org_id: str, corp_email: str):
    emp_id = hashlib.sha256(f"corp:email:{org_id}:{corp_email}".encode()).hexdigest()
    return emp_id


def convert_to_valid_pem(private_key: str):
    if private_key.startswith("-----BEGIN"):
        return private_key
    pieces = []
    pieces.append("-----BEGIN EC PRIVATE KEY-----")
    last_idx = 0
    for x in range(64, len(private_key), 64):
        pieces.append(private_key[last_idx:x])
        last_idx = x
    pieces.append(private_key[last_idx:])
    pieces.append("-----END EC PRIVATE KEY-----")
    return "\n".join(pieces)


def verify_dataset(private_key: str, dataset: str, signature: str):
    try:
        private_key_pem = convert_to_valid_pem(private_key)
        sk = ecdsa.SigningKey.from_pem(private_key_pem)
        vk = sk.get_verifying_key()
        sig_bytes = base64.b64decode(signature)
        dataset = dataset.encode("utf-8")
        result = vk.verify(
            sig_bytes,
            dataset,
            hashfunc=hashlib.sha256,
            sigdecode=ecdsa.util.sigdecode_der,
        )
        return result
    except BadSignatureError:
        return False
    except Exception as _:
        raise PrivateKeyParseError(_)


def batch_list(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i : i + batch_size]



from pureauth_pysdk.dto import PureAUTHServerConfigDTO

from .employee import Employee
from .organization import Organization


class V2:
    def __init__(self, config: PureAUTHServerConfigDTO):
        self.config = config
        self.organization = Organization(self.config)
        self.employees = Employee(self.config, self.organization)



from .dto import PureAUTHServerConfigDTO
from .errors import AccessTokenValidationError
from .sync import Sync


class Pureauth:
    def __init__(
        self,
        organization_id: str,
        access_token: str,
        base_url: str = "https://live.pureauth.io",
        private_key: str = None,
    ):
        api_version = "v1"  # Force v1 until v2 is complete.
        self.config = PureAUTHServerConfigDTO(
            organization_id=organization_id,
            access_token=access_token,
            base_url=base_url if base_url[-1] != "/" else base_url[:-1],
            api_version=api_version,
            private_key=private_key,
        )
        self.validate_accesstoken()
        self.sync = Sync(config=self.config)

    def validate_accesstoken(self) -> bool:
        # raise AccessTokenValidationError
        pass



class PureAUTHApiError(Exception):
    pass


class DatasetListApiError(Exception):
    pass


class PublicKeyUploadApiError(Exception):
    pass


class DatasetCsvApiError(Exception):
    pass


class StatusApiError(Exception):
    pass


class DatasetFormatError(Exception):
    pass


class UserAddApiError(Exception):
    pass


class UserUpdateApiError(Exception):
    pass


class WelcomeEmailApiError(Exception):
    pass


class ActivateEmployeeApiError(Exception):
    pass


class DeactivateEmployeeApiError(Exception):
    pass


class OrganizationGroupsApiError(Exception):
    pass


class GroupAttachApiError(Exception):
    pass


class RoleAttachApiError(Exception):
    pass


class OrganizationLogsApiError(Exception):
    pass


class OrganizationLogsDateFormatError(Exception):
    pass


class JSONDecodeError(Exception):
    pass


class SignatureApiError(Exception):
    pass


class AutoOSMFetchApiError(Exception):
    pass



import json
from collections import OrderedDict
from dataclasses import dataclass
from datetime import datetime
from .errors import EmployeeDTOMissingDataError

from pureauth_pysdk.dto import PureAUTHServerConfigDTO


class ApiUri:
    def __init__(self, config: PureAUTHServerConfigDTO):
        self.base_url = config.base_url
        self.api_version = "v1"
        self.api_base = f"{config.base_url}/api/{config.api_version}/organizations"

    def fetch_csv_url(self) -> str:
        return f"{self.api_base}/datasets/csv-template"

    def public_key_upload_url(self) -> str:
        return f"{self.api_base}/publickey"

    def dataset_url(self) -> str:
        return f"{self.api_base}/datasets/?include=all"

    def employee_add_url(self) -> str:
        return f"{self.api_base}/employees/"

    def employee_status_url(self, employee_id: str) -> str:
        return f"{self.api_base}/employees/{employee_id}/profile/status"

    def employee_activate_url(self, employee_id: str) -> str:
        return f"{self.api_base}/employees/{employee_id}/profile/activate"

    def employee_deactivate_url(self, employee_id: str) -> str:
        return f"{self.api_base}/employees/{employee_id}/profile/deactivate"

    def employee_update_url(self, employee_id: str) -> str:
        return f"{self.api_base}/employees/{employee_id}"

    def employee_group_attach_url(self, employee_id: str) -> str:
        return f"{self.api_base}/employees/{employee_id}/groups"

    def employee_signature_url(self, employee_id: str) -> str:
        return f"{self.api_base}/employees/{employee_id}/signatures"

    def group_list_url(self) -> str:
        return f"{self.api_base}/groups"

    def employee_role_attach_url(self, employee_id: str) -> str:
        return f"{self.api_base}/employees/{employee_id}/roles"

    def role_list_url(self) -> str:
        return f"{self.api_base}/roles"

    def welcome_email_url(self, employee_id: str) -> str:
        return f"{self.api_base}/employees/{employee_id}/profile/email-details"

    def fetch_logs_url(self) -> str:
        return f"{self.base_url}/api/v1/audit_logs/"

    def fetch_logs_url_filtered(self, from_date: datetime, to_date: datetime) -> str:
        d1 = from_date.strftime("%d/%m/%Y")
        d2 = to_date.strftime("%d/%m/%Y")
        return (
            f"{self.base_url}/api/v1/audit_logs/?from_date={d1}&to_date={d2}&order=desc"
        )


@dataclass
class EmployeeStatusDTO:
    is_active: bool
    osm: str
    onboarded: bool


@dataclass
class PrimaryDatasetMappingDTO:
    full_name: str = "Full Name"
    primary_email: str = "Personal Email"
    corporate_email: str = "Corporate Email"
    phone_number: str = "Phone Number"


@dataclass(init=False, repr=True, eq=True)
class EmployeePrimaryDatasetDTO:
    full_name: str
    primary_email: str
    corporate_email: str
    phone_number: str
    user_dataset: dict

    def __init__(
        self,
        full_name: str,
        primary_email: str,
        corporate_email: str,
        phone_number: str,
        mapping: PrimaryDatasetMappingDTO = PrimaryDatasetMappingDTO(),
    ):
        # Validate all data exists
        if not full_name:
            raise EmployeeDTOMissingDataError(
                "Missing required dataset attribute: full_name"
            )
        if not primary_email:
            raise EmployeeDTOMissingDataError(
                "Missing required dataset attribute: primary_email"
            )
        if not corporate_email:
            raise EmployeeDTOMissingDataError(
                "Missing required dataset attribute: corporate_email"
            )
        if not phone_number:
            raise EmployeeDTOMissingDataError(
                "Missing required dataset attribute: phone_number"
            )
        # Normalize data
        self.full_name = full_name.lower().strip()
        self.primary_email = primary_email.lower().strip()
        self.corporate_email = corporate_email.lower().strip()
        self.phone_number = (
            phone_number.lower().strip().replace("-", "").replace(" ", "")
        )
        self.user_dataset = {
            mapping.full_name: self.full_name,
            mapping.corporate_email: self.corporate_email,
            mapping.primary_email: self.primary_email,
            mapping.phone_number: self.phone_number,
        }


@dataclass(init=False, repr=True, eq=True)
class EmployeeSecondaryDatasetDTO:
    username: str
    user_principal_name: str
    employee_id: str
    object_id: str
    employee_type: str
    otherattribute1: dict

    def __init__(
        self,
        username: str = "",
        user_principal_name: str = "",
        employee_id: str = "",
        object_id: str = "",
        employee_type: str = "",
        otherattribute1: str = "",
    ) -> None:
        self.username = username.strip() if username is not None else ""
        self.user_principal_name = (
            user_principal_name.strip() if user_principal_name is not None else ""
        )
        self.employee_id = employee_id.strip() if employee_id is not None else ""
        self.object_id = object_id.strip() if object_id is not None else ""
        self.employee_type = employee_type.strip() if employee_type is not None else ""
        self.otherattribute1 = (
            otherattribute1.strip() if otherattribute1 is not None else ""
        )
        self.serialized = json.dumps(
            OrderedDict(
                [
                    ("Username", self.username),
                    ("UserPrincipalName", self.user_principal_name),
                    ("EmployeeID", self.employee_id),
                    ("ObjectID", self.object_id),
                    ("EmployeeType", self.employee_type),
                    ("OtherAttribute1", self.otherattribute1),
                ]
            )
        )


@dataclass
class EmployeeDatasetDTO:
    primary_dataset: EmployeePrimaryDatasetDTO
    secondary_dataset: EmployeeSecondaryDatasetDTO


@dataclass
class LogsDTO:
    log_id: str
    actor_id: str
    actor_role: str
    actor_deanonymized: str
    created_at: str
    event: str
    ip_address: str
    message: str
    outcome: str
